require 'httparty'

module ConfigStore
	class API

		include HTTParty

		include AccountAPIMethods
		include NamespaceAPIMethods
		include RecordAPIMethods
		include TokenAPIMethods

		default_timeout 30

		def initialize(token:, base_url:)
			@token = token
			@options = {
				base_uri: base_url,
				headers: headers,
			}
		end

		private

		def headers
			return {
				"X-Token" => @token,
				'Content-Type' => 'application/json',
			}
		end

		def with_response_handling
			response = begin
				yield
			rescue ::Timeout::Error # includes ::Net::OpenTimeout and ::Net::ReadTimeout
				raise ConfigStore::Timeout
			end
			handle_error(response) unless response.success?
			return response.parsed_response
		end

		def handle_error(response)
			raise ConfigStore::HTTPError.new(status_code: response.code, response: response.parsed_response)
		end

	end
end
