require "base64"

module ConfigStore
	class Record < Model

		ATTRIBUTES = {
			uuid: TypedAttrAccessor::TYPE_STRING,
			key: TypedAttrAccessor::TYPE_STRING,
			raw_value: TypedAttrAccessor::TYPE_BASE64,
			metadata: TypedAttrAccessor::TYPE_HASH,
			ttl: TypedAttrAccessor::TYPE_INTEGER,
			created_at: TypedAttrAccessor::TYPE_DATE_TIME,
			updated_at: TypedAttrAccessor::TYPE_DATE_TIME,
			last_access: TypedAttrAccessor::TYPE_DATE_TIME,
			value: TypedAttrAccessor::TYPE_STRING,
		}.deep_freeze

		API_ATTRIBUTE_LIST = [:uuid, :key, :raw_value, :metadata, :ttl, :created_at, :updated_at, :last_access].deep_freeze

		attributes_hash.each do |name, type_alias|
			typed_attr_accessor(name, type_alias)
		end

		# keep raw_value and value in sync, allow calling super for typed_attr_accessor
		module SetValueSyncer
			def raw_value=(new_raw_value)
				super(new_raw_value)
				new_value = Base64.strict_decode64(new_raw_value) unless new_raw_value.nil?
				instance_variable_set("@value", new_value)
				return new_raw_value
			end

			def value=(new_value)
				super(new_value)
				new_raw_value = Base64.strict_encode64(new_value) unless new_value.nil?
				instance_variable_set("@raw_value", new_raw_value)
				return new_value
			end

			def initialize(**args)
				if args.has_key?(:raw_value) && args.has_key?(:value)
					raise ArgumentError, "only one of the :raw_value and :value attributes should be provided"
				end
				super(**args)
				self.raw_value = args[:raw_value] if args.has_key?(:raw_value)
				self.value = args[:value] if args.has_key?(:value)
			end
		end
		private_constant :SetValueSyncer
		prepend SetValueSyncer

	end
end
